package org.toastedtruth.json.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;

import org.toastedtruth.util.JSONObjectTreeModelAdapter;

public class MainPanel extends JPanel {
    private static final long serialVersionUID = 2500275779325190676L;
    private JTextArea syntaxArea;
    private JTextField nameArea;
    private JEditorPane valueArea;
    private JTree jsonTree;
    public final JSONObjectTreeModelAdapter t = new JSONObjectTreeModelAdapter(Main.FILE_NAME);

    public MainPanel() {
        init();
    }

    public String getCurrentName() {
        return nameArea.getText();
    }

    public String getCurrentValue() {
        return valueArea.getText();
    }

    public void setSyntax(String syntax) {
        syntaxArea.setText(syntax);
    }

    public void setTreeModel(TreeModel model) {
        jsonTree.setModel(model);
    }

    public void expandTree() {
        for (int i = 0; i < jsonTree.getRowCount(); i++) {
            jsonTree.expandRow(i);
        }
    }

    public void collapseTree() {
        for (int i = 0; i < jsonTree.getRowCount(); i++) {
            jsonTree.collapseRow(i);
        }
    }

    public void redrawTree() {
        TreeModel currentModel = jsonTree.getModel();
        if (currentModel == null) {
            MutableTreeNode node = new DefaultMutableTreeNode(Main.FILE_NAME);
            node.insert(new DefaultMutableTreeNode("Empty"), node.getChildCount());
            jsonTree.setModel(new DefaultTreeModel(node));
            return;
        }
        jsonTree.setModel(t.toTreeModel());
    }

    private void init() {
        this.setLayout(new BorderLayout(0, 0));

        JSplitPane splitPane = new JSplitPane();
        splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        splitPane.setBorder(null);
        splitPane.setContinuousLayout(true);
        splitPane.setResizeWeight(0.7);
        this.add(splitPane);

        jsonTree = new JTree();
        jsonTree.setEditable(false);
        jsonTree.setModel(null);
        this.redrawTree();

        splitPane.setRightComponent(new JScrollPane(jsonTree));

        JPanel innerPanel = new JPanel();
        splitPane.setLeftComponent(innerPanel);
        innerPanel.setLayout(new BorderLayout(0, 0));

        JPanel topPanel = new JPanel();
        FlowLayout fl_topPanel = (FlowLayout) topPanel.getLayout();
        fl_topPanel.setAlignment(FlowLayout.LEFT);
        innerPanel.add(topPanel, BorderLayout.NORTH);

        ButtonListener btnListener = new ButtonListener(this);
        JButton btnAdd = new JButton("+");
        btnAdd.setActionCommand("add");
        btnAdd.addActionListener(btnListener);
        topPanel.add(btnAdd);

        JButton btnRemove = new JButton("-");
        btnRemove.setActionCommand("remove");
        topPanel.add(btnRemove);

        JButton btnExpandAll = new JButton("Expand All >>");
        btnExpandAll.setActionCommand("expand");
        btnExpandAll.addActionListener(btnListener);
        topPanel.add(btnExpandAll);

        JButton btnCollapseAll = new JButton("Collapse All <<");
        btnCollapseAll.setActionCommand("collapse");
        btnCollapseAll.addActionListener(btnListener);
        topPanel.add(btnCollapseAll);

        JPanel editorPanel = new JPanel();
        innerPanel.add(editorPanel, BorderLayout.CENTER);
        GridBagLayout gbl_editorPanel = new GridBagLayout();
        gbl_editorPanel.columnWidths = new int[] { 0, 0 };
        gbl_editorPanel.columnWeights = new double[] { 1.0, 1.0 };
        gbl_editorPanel.rowHeights = new int[] { 0, 0, 0, 0, 0, 0, 0 };
        gbl_editorPanel.rowWeights = new double[] { 0, 0, 0, 0, 0, 0.0, 1.0 };
        editorPanel.setLayout(gbl_editorPanel);

        JPanel divider = new JPanel();
        divider.setBackground(SystemColor.scrollbar);
        divider.setLayout(null);
        GridBagConstraints gbc_divider = new GridBagConstraints();
        gbc_divider.gridwidth = 2;
        gbc_divider.insets = new Insets(0, 0, 5, 0);
        gbc_divider.fill = GridBagConstraints.BOTH;
        gbc_divider.gridx = 0;
        gbc_divider.gridy = 0;
        editorPanel.add(divider, gbc_divider);

        nameArea = new JTextField();
        nameArea.setBorder(LineBorder.createGrayLineBorder());
        GridBagConstraints gbc_txtObjectName = new GridBagConstraints();
        gbc_txtObjectName.insets = new Insets(2, 5, 5, 5);
        gbc_txtObjectName.anchor = GridBagConstraints.NORTHWEST;
        gbc_txtObjectName.fill = GridBagConstraints.HORIZONTAL;
        gbc_txtObjectName.gridx = 0;
        gbc_txtObjectName.gridy = 2;
        editorPanel.add(nameArea, gbc_txtObjectName);
        nameArea.setColumns(10);

        JLabel lblNode = new JLabel("Node:");
        GridBagConstraints gbc_lblNode = new GridBagConstraints();
        gbc_lblNode.insets = new Insets(0, 6, 5, 5);
        gbc_lblNode.anchor = GridBagConstraints.WEST;
        gbc_lblNode.gridx = 0;
        gbc_lblNode.gridy = 1;
        editorPanel.add(lblNode, gbc_lblNode);

        JLabel lblValueType = new JLabel("Value Type:");
        GridBagConstraints gbc_lblValueEditor = new GridBagConstraints();
        gbc_lblValueEditor.insets = new Insets(3, 6, 5, 5);
        gbc_lblValueEditor.anchor = GridBagConstraints.WEST;
        gbc_lblValueEditor.gridx = 0;
        gbc_lblValueEditor.gridy = 3;
        editorPanel.add(lblValueType, gbc_lblValueEditor);

        JLabel lblValue = new JLabel("Value:");
        GridBagConstraints gbc_lblValue = new GridBagConstraints();
        gbc_lblValue.insets = new Insets(3, 6, 5, 5);
        gbc_lblValue.anchor = GridBagConstraints.WEST;
        gbc_lblValue.gridx = 0;
        gbc_lblValue.gridy = 5;
        editorPanel.add(lblValue, gbc_lblValue);

        JLabel lblSyntaxEditor = new JLabel("JSON:");
        GridBagConstraints gbc_lblSyntaxEditor = new GridBagConstraints();
        gbc_lblSyntaxEditor.insets = new Insets(0, 1, 5, 0);
        gbc_lblSyntaxEditor.anchor = GridBagConstraints.WEST;
        gbc_lblSyntaxEditor.gridx = 1;
        gbc_lblSyntaxEditor.gridy = 1;
        editorPanel.add(lblSyntaxEditor, gbc_lblSyntaxEditor);

        syntaxArea = new JTextArea();
        syntaxArea.setBorder(new LineBorder(Color.GRAY));
        syntaxArea.setEnabled(false);
        syntaxArea.setEditable(false);
        syntaxArea.setWrapStyleWord(true);
        syntaxArea.setLineWrap(true);

        JScrollPane syntaxScroll = new JScrollPane(syntaxArea);
        syntaxScroll.setBorder(new EmptyBorder(0, 0, 0, 0));
        GridBagConstraints gbc_syntaxEditor = new GridBagConstraints();
        gbc_syntaxEditor.insets = new Insets(2, 0, 5, 5);
        gbc_syntaxEditor.gridheight = 5;
        gbc_syntaxEditor.fill = GridBagConstraints.BOTH;
        gbc_syntaxEditor.gridx = 1;
        gbc_syntaxEditor.gridy = 2;
        editorPanel.add(syntaxScroll, gbc_syntaxEditor);

        JComboBox cmboValueType = new JComboBox();
        GridBagConstraints gbc_cmboValueType = new GridBagConstraints();
        gbc_cmboValueType.insets = new Insets(2, 5, 5, 5);
        gbc_cmboValueType.fill = GridBagConstraints.HORIZONTAL;
        gbc_cmboValueType.gridx = 0;
        gbc_cmboValueType.gridy = 4;
        editorPanel.add(cmboValueType, gbc_cmboValueType);

        JScrollPane valueScroll = new JScrollPane();
        GridBagConstraints gbc_valueScroll = new GridBagConstraints();
        gbc_valueScroll.insets = new Insets(2, 5, 5, 5);
        gbc_valueScroll.fill = GridBagConstraints.BOTH;
        gbc_valueScroll.gridx = 0;
        gbc_valueScroll.gridy = 6;
        editorPanel.add(valueScroll, gbc_valueScroll);

        valueArea = new JEditorPane();
        valueScroll.setViewportView(valueArea);

        JPanel panel_2 = new JPanel();
        panel_2.setLayout(new BorderLayout(0, 0));
    }
}
