package org.toastedtruth.json.gui;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;

import org.toastedtruth.json.gui.listener.FileMenuListener;

public class MenuBar extends JMenuBar {

    private final Frame mainFrame;
    private final MainPanel mainPanel;
    private static final long serialVersionUID = 5525549373837432927L;

    public MenuBar(Frame mainFrame, MainPanel mainPanel) {
        this.mainFrame = mainFrame;
        this.mainPanel = mainPanel;
        init();
    }

    private void init() {

        JMenu fileMenu = new JMenu("File");
        this.add(fileMenu);

        FileMenuListener fileListener = new FileMenuListener(mainFrame, mainPanel);

        JMenuItem fileNew = new JMenuItem("New");
        fileNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
        fileMenu.add(fileNew);

        JSeparator sep1 = new JSeparator();
        fileMenu.add(sep1);

        JMenuItem fileOpen = new JMenuItem("Open...");
        fileOpen.setActionCommand("open");
        fileOpen.addActionListener(fileListener);
        fileOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
        fileMenu.add(fileOpen);

        JMenuItem fileSave = new JMenuItem("Save");
        fileSave.setActionCommand("save");
        fileSave.addActionListener(fileListener);
        fileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        fileMenu.add(fileSave);

        JMenuItem fileSaveAs = new JMenuItem("Save As...");
        fileSaveAs.setActionCommand("saveas");
        fileSaveAs.addActionListener(fileListener);
        fileSaveAs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
        fileMenu.add(fileSaveAs);

        JSeparator sep2 = new JSeparator();
        fileMenu.add(sep2);

        JMenuItem fileExit = new JMenuItem("Exit");
        fileExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mainFrame.dispose();
                System.exit(0);
            }
        });
        fileMenu.add(fileExit);

        JMenu editMenu = new JMenu("Edit");
        this.add(editMenu);

        JMenuItem editUndo = new JMenuItem("Undo");
        editUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
        editMenu.add(editUndo);

        JMenuItem editRedo = new JMenuItem("Redo");
        editRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));
        editMenu.add(editRedo);

        JSeparator sep3 = new JSeparator();
        editMenu.add(sep3);

        JMenuItem editCut = new JMenuItem("Cut");
        editCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
        editMenu.add(editCut);

        JMenuItem editCopy = new JMenuItem("Copy");
        editCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
        editMenu.add(editCopy);

        JMenuItem editPaste = new JMenuItem("Paste");
        editPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
        editMenu.add(editPaste);

        JSeparator sep4 = new JSeparator();
        editMenu.add(sep4);

        JMenuItem editViewRaw = new JMenuItem("View Raw");
        editMenu.add(editViewRaw);

        JMenuItem editModifyRaw = new JMenuItem("Modify Raw");
        editMenu.add(editModifyRaw);
    }
}
