package org.toastedtruth.json.gui.listener;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.json.JSONException;
import org.json.JSONObject;
import org.toastedtruth.json.gui.JsonFilter;
import org.toastedtruth.json.gui.Main;
import org.toastedtruth.json.gui.MainPanel;

public class FileMenuListener implements ActionListener {

    private final Frame mainFrame;
    private final MainPanel mainPanel;
    private File loadedFile;

    public FileMenuListener(Frame mainFrame, MainPanel mainPanel) {
        this.mainFrame = mainFrame;
        this.mainPanel = mainPanel;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("open")) {
            open();
            return;
        }

        if (e.getActionCommand().equals("save")) {
            if (loadedFile == null) saveAs();
            else save();
            return;
        }

        if (e.getActionCommand().equals("saveas")) {
            saveAs();
            return;
        }
    }

    private void open() {
        JFileChooser open = new JFileChooser();
        open.setDialogTitle("Open");
        open.setAcceptAllFileFilterUsed(false);
        open.setFileSelectionMode(JFileChooser.FILES_ONLY);
        open.setFileFilter(new JsonFilter());
        if (open.showOpenDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
            try {
                loadedFile = open.getSelectedFile();

                BufferedReader br = new BufferedReader(new FileReader(loadedFile));
                StringBuilder b = new StringBuilder();
                String line = null;
                while ((line = br.readLine()) != null) {
                    b.append(line);
                }
                br.close();
                try {
                    mainPanel.t.setRoot(new JSONObject(b.toString()));
                } catch (JSONException ex) {
                    JOptionPane.showMessageDialog(mainFrame, "Error in JSON file:\n" + ex.getLocalizedMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                }
                mainPanel.t.setName(loadedFile.getName());
                mainPanel.setSyntax(mainPanel.t.toJsonObject().toString(2));
                mainPanel.redrawTree();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    private void save() {
        try {
            FileWriter out = new FileWriter(loadedFile);
            mainPanel.t.toJsonObject().write(out);
            out.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        mainPanel.redrawTree();
    }

    private void saveAs() {
        JFileChooser save = new JFileChooser();
        save.setDialogTitle("Save As...");
        save.setAcceptAllFileFilterUsed(false);
        save.setFileSelectionMode(JFileChooser.FILES_ONLY);
        save.setSelectedFile(new File("New JSON File.json"));
        save.setFileFilter(new JsonFilter());
        if (save.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
            JSONObject object = mainPanel.t.toJsonObject();
            File file = save.getSelectedFile();
            if (!file.getName().toLowerCase().endsWith(".json")) {
                File absolute = file.getParentFile();
                String name = file.getName();
                file = new File(absolute, name + ".json");
                save.setSelectedFile(file);
            }
            if (file.exists()) {
                if (!(JOptionPane.showConfirmDialog(save, file.getName() + " already exists. Overwrite?", "Overwrite File?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)) return;
                if (!file.delete()) System.out.println("Not deleted :o");
            }
            try {
                FileWriter out = new FileWriter(file);
                object.write(out);
                out.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            Main.FILE_NAME = file.getName();
            mainPanel.redrawTree();
        }
    }
}
