package org.toastedtruth.util;

import java.util.Enumeration;
import java.util.Set;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;

import org.json.JSONArray;
import org.json.JSONObject;

public class JSONObjectTreeModelAdapter {

    private String name;
    private JSONObject root = new JSONObject();

    public JSONObjectTreeModelAdapter(String name) {
        this.name = name;
    }

    public void setName(String name) {
        this.name = name;
        JSONObject o = new JSONObject();
        for (Object key : root.keySet()) {
            o.put(key.toString(), root.get(key.toString()));
        }
        root = o;
    }

    public void setRoot(JSONObject root) {
        this.root = root;
    }

    public void add(String key, Object object) {
        String[] nodeSplit = key.split("\\.");

        JSONObject node = root;
        for (int i = 0; i < nodeSplit.length; i++) {
            if (i == nodeSplit.length - 1) {
                node.put(nodeSplit[i], object);
                break;
            } else {
                JSONObject child;
                if (node.has(nodeSplit[i]) && node.get(nodeSplit[i]) != null && node.get(nodeSplit[i]) instanceof JSONObject) {
                    child = node.getJSONObject(nodeSplit[i]);
                } else if (node.has(nodeSplit[i]) && node.get(nodeSplit[i]) != null && node.get(nodeSplit[i]) instanceof JSONArray) {
                    child = new JSONObject();
                    child.put(nodeSplit[i], node.getJSONArray(nodeSplit[i]));
                } else {
                    child = new JSONObject();
                }
                node.put(nodeSplit[i], child);
                node = child;
            }
        }
    }

    public JSONObject toJsonObject() {
        return root;
    }

    public TreeModel toTreeModel() {
        TreeNode node = toTreeNode(name, root);
        DefaultTreeModel model = new DefaultTreeModel(node);
        return model;
    }

    public TreeNode toTreeNode(String rootKey, JSONObject json) {
        DefaultMutableTreeNode main = new DefaultMutableTreeNode(rootKey);
        DefaultMutableTreeNode node = main;

        Set<String> keys = json.keySet();
        for (String key : keys) {
            int index = childIndex(node, key);
            if (index < 0) {
                DefaultMutableTreeNode newChild = new DefaultMutableTreeNode(key);
                Object o = json.get(key);
                if (o instanceof JSONObject) {
                    newChild = (DefaultMutableTreeNode) toTreeNode(key, json.getJSONObject(key));
                    node.insert(newChild, node.getChildCount());
                    node = main;
                } else {
                    newChild = new DefaultMutableTreeNode(key + ": " + o.toString());
                    node.insert(newChild, node.getChildCount());
                }
            } else {
                node = (DefaultMutableTreeNode) node.getChildAt(index);
            }
        }
        return main;
    }

    private int childIndex(DefaultMutableTreeNode node, Object childValue) {
        Enumeration<DefaultMutableTreeNode> children = node.children();
        DefaultMutableTreeNode child = null;
        int index = -1;
        while (children.hasMoreElements() && index < 0) {
            child = children.nextElement();
            if (child.getUserObject() != null && childValue.equals(child.getUserObject())) {
                index = node.getIndex(child);
            }
        }
        return index;
    }
}
